@@ -8,7 +8,7 @@ use std::ffi;
 
 /// Provides access to a Nitrokey device
 #[derive(Debug, structopt::StructOpt)]
-#[structopt(name = "nitrocli")]
+#[structopt(name = "nitrocli", no_version)]
 pub struct Args {
   /// Increases the log level (can be supplied multiple times)
   #[structopt(short, long, global = true, parse(from_occurrences))]
@@ -103,7 +103,7 @@ Command! {
     /// Accesses the password safe
     Pws(PwsArgs) => |ctx, args: PwsArgs| args.subcmd.execute(ctx),
     /// Performs a factory reset
-    Reset => crate::commands::reset,
+    Reset(ResetArgs) => |ctx, args: ResetArgs| crate::commands::reset(ctx, args.only_aes_key),
     /// Prints the status of the connected Nitrokey device
     Status => crate::commands::status,
     /// Interacts with the device's unencrypted volume
@@ -394,9 +394,19 @@ Command! {PwsCommand, [
   Get(PwsGetArgs) => |ctx, args: PwsGetArgs| {
     crate::commands::pws_get(ctx, args.slot, args.name, args.login, args.password, args.quiet)
   },
-  /// Writes a password safe slot
-  Set(PwsSetArgs) => |ctx, args: PwsSetArgs| {
-    crate::commands::pws_set(ctx, args.slot, &args.name, &args.login, &args.password)
+  /// Adds a new password safe slot
+  Add(PwsAddArgs) => |ctx, args: PwsAddArgs| {
+    crate::commands::pws_add(ctx, &args.name, &args.login, &args.password, args.slot)
+  },
+  /// Updates a password safe slot
+  Update(PwsUpdateArgs) => |ctx, args: PwsUpdateArgs| {
+    crate::commands::pws_update(
+      ctx,
+      args.slot,
+      args.name.as_deref(),
+      args.login.as_deref(),
+      args.password.as_deref()
+    )
   },
   /// Prints the status of the password safe slots
   Status(PwsStatusArgs) => |ctx, args: PwsStatusArgs| crate::commands::pws_status(ctx, args.all),
@@ -427,15 +437,33 @@ pub struct PwsGetArgs {
 }
 
 #[derive(Debug, PartialEq, structopt::StructOpt)]
-pub struct PwsSetArgs {
-  /// The PWS slot to write
-  pub slot: u8,
+pub struct PwsAddArgs {
   /// The name to store on the slot
   pub name: String,
   /// The login to store on the slot
   pub login: String,
   /// The password to store on the slot
   pub password: String,
+  /// The number of the slot to write
+  ///
+  /// If this option is not set, the first unprogrammed slot is used.
+  #[structopt(short, long)]
+  pub slot: Option<u8>,
+}
+
+#[derive(Debug, PartialEq, structopt::StructOpt)]
+pub struct PwsUpdateArgs {
+  /// The PWS slot to update
+  pub slot: u8,
+  /// The new name to store on the slot
+  #[structopt(short, long)]
+  pub name: Option<String>,
+  /// The new login to store on the slot
+  #[structopt(short, long)]
+  pub login: Option<String>,
+  /// The new password to store on the slot
+  #[structopt(short, long)]
+  pub password: Option<String>,
 }
 
 #[derive(Debug, PartialEq, structopt::StructOpt)]
@@ -445,6 +473,13 @@ pub struct PwsStatusArgs {
   pub all: bool,
 }
 
+#[derive(Debug, PartialEq, structopt::StructOpt)]
+pub struct ResetArgs {
+  /// Only build a new AES key instead of performing a full factory reset.
+  #[structopt(long)]
+  pub only_aes_key: bool,
+}
+
 #[derive(Debug, PartialEq, structopt::StructOpt)]
 pub struct UnencryptedArgs {
   #[structopt(subcommand)]
