@@ -1,6 +1,6 @@
 // commands.rs
 
-// Copyright (C) 2018-2020 The Nitrocli Developers
+// Copyright (C) 2018-2021 The Nitrocli Developers
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 use std::borrow;
@@ -10,7 +10,6 @@ use std::ffi;
 use std::fmt;
 use std::fs;
 use std::io;
-use std::mem;
 use std::ops;
 use std::ops::Deref as _;
 use std::path;
@@ -36,6 +35,12 @@ use crate::Context;
 
 const NITROCLI_EXT_PREFIX: &str = "nitrocli-";
 
+const OTP_NAME_LENGTH: usize = 15;
+
+const PWS_NAME_LENGTH: usize = 11;
+const PWS_LOGIN_LENGTH: usize = 32;
+const PWS_PASSWORD_LENGTH: usize = 20;
+
 /// Set `libnitrokey`'s log level based on the execution context's verbosity.
 fn set_log_level(ctx: &mut Context<'_>) {
   let log_lvl = match ctx.config.verbosity {
@@ -378,12 +383,65 @@ fn print_storage_status(
   Ok(())
 }
 
-/// Query and pretty print the status that is common to all Nitrokey devices.
-fn print_status(ctx: &mut Context<'_>, device: &nitrokey::DeviceWrapper<'_>) -> anyhow::Result<()> {
-  let serial_number = device
-    .get_serial_number()
-    .context("Could not query the serial number")?;
+/// Read a value from stdin if the given string is set to "-".
+fn value_or_stdin<'s>(ctx: &mut Context<'_>, s: &'s str) -> anyhow::Result<borrow::Cow<'s, str>> {
+  if s == "-" {
+    let mut s = String::new();
+    let _ = ctx
+      .stdin
+      .read_to_string(&mut s)
+      .context("Failed to read from stdin")?;
+    Ok(borrow::Cow::from(s))
+  } else {
+    Ok(borrow::Cow::from(s))
+  }
+}
 
+/// Validate the length of strings provided by the user.
+///
+/// The input must be a slice of tuples of the name of the string, the string itself and the
+/// maximum length.
+fn ensure_string_lengths(data: &[(&str, &str, usize)]) -> anyhow::Result<()> {
+  let mut invalid_strings = Vec::new();
+  for (label, value, max_length) in data {
+    let length = value.as_bytes().len();
+    if length > *max_length {
+      invalid_strings.push((label, length, max_length));
+    }
+  }
+  match invalid_strings.len() {
+    0 => Ok(()),
+    1 => {
+      let (label, length, max_length) = invalid_strings[0];
+      Err(anyhow::anyhow!(
+        "The provided {} is too long (actual length: {} bytes, maximum length: {} bytes)",
+        label,
+        length,
+        max_length
+      ))
+    }
+    _ => {
+      let mut msg = String::from("Multiple provided strings are too long:");
+      for (label, length, max_length) in invalid_strings {
+        msg.push_str(&format!(
+          "\n  {} (actual length: {} bytes, maximum length: {} bytes)",
+          label, length, max_length
+        ));
+      }
+      Err(anyhow::anyhow!(msg))
+    }
+  }
+}
+
+/// Pretty print the status that is common to all Nitrokey devices.
+fn print_status(
+  ctx: &mut Context<'_>,
+  model: nitrokey::Model,
+  serial_number: nitrokey::SerialNumber,
+  firmware_version: nitrokey::FirmwareVersion,
+  user_retry_count: u8,
+  admin_retry_count: u8,
+) -> anyhow::Result<()> {
   println!(
     ctx,
     r#"Status:
@@ -392,36 +450,62 @@ fn print_status(ctx: &mut Context<'_>, device: &nitrokey::DeviceWrapper<'_>) ->
   firmware version:  {fwv}
   user retry count:  {urc}
   admin retry count: {arc}"#,
-    model = device.get_model(),
+    model = model,
     id = serial_number,
-    fwv = device
-      .get_firmware_version()
-      .context("Failed to retrieve firmware version")?,
-    urc = device
-      .get_user_retry_count()
-      .context("Failed to retrieve user retry count")?,
-    arc = device
-      .get_admin_retry_count()
-      .context("Failed to retrieve admin retry count")?,
+    fwv = firmware_version,
+    urc = user_retry_count,
+    arc = admin_retry_count,
   )?;
 
-  if let nitrokey::DeviceWrapper::Storage(device) = device {
-    let status = device
-      .get_storage_status()
-      .context("Failed to retrieve storage status")?;
-    let sd_card_usage = device
-      .get_sd_card_usage()
-      .context("Failed to retrieve SD card usage")?;
-
-    print_storage_status(ctx, &status, &sd_card_usage)
-  } else {
-    Ok(())
-  }
+  Ok(())
 }
 
 /// Inquire the status of the nitrokey.
 pub fn status(ctx: &mut Context<'_>) -> anyhow::Result<()> {
-  with_device(ctx, |ctx, device| print_status(ctx, &device))
+  with_device(ctx, |ctx, device| {
+    if let nitrokey::DeviceWrapper::Storage(device) = device {
+      // TODO: Extract serial number from storage status, see
+      //       https://todo.sr.ht/~ireas/nitrokey-rs/1
+      let serial_number = device
+        .get_serial_number()
+        .context("Failed to retrieve serial number")?;
+      let status = device
+        .get_storage_status()
+        .context("Failed to retrieve storage status")?;
+
+      print_status(
+        ctx,
+        device.get_model(),
+        serial_number,
+        status.firmware_version,
+        status.user_retry_count,
+        status.admin_retry_count,
+      )?;
+
+      let sd_card_usage = device
+        .get_sd_card_usage()
+        .context("Failed to retrieve SD card usage")?;
+      print_storage_status(ctx, &status, &sd_card_usage)
+    } else {
+      let status = device
+        .get_status()
+        .context("Could not query the device status")?;
+      let user_retry_count = device
+        .get_user_retry_count()
+        .context("Failed to retrieve user retry count")?;
+      let admin_retry_count = device
+        .get_admin_retry_count()
+        .context("Failed to retrieve admin retry count")?;
+      print_status(
+        ctx,
+        device.get_model(),
+        status.serial_number,
+        status.firmware_version,
+        user_retry_count,
+        admin_retry_count,
+      )
+    }
+  })
 }
 
 /// List the attached Nitrokey devices.
@@ -514,7 +598,7 @@ pub fn fill(ctx: &mut Context<'_>, attach: bool) -> anyhow::Result<()> {
 }
 
 /// Perform a factory reset.
-pub fn reset(ctx: &mut Context<'_>) -> anyhow::Result<()> {
+pub fn reset(ctx: &mut Context<'_>, only_aes_key: bool) -> anyhow::Result<()> {
   with_device(ctx, |ctx, mut device| {
     let pin_entry = pinentry::PinEntry::from(args::PinType::Admin, &device)?;
 
@@ -523,20 +607,28 @@ pub fn reset(ctx: &mut Context<'_>) -> anyhow::Result<()> {
     pinentry::clear(&pin_entry).context("Failed to clear cached secret")?;
 
     try_with_pin(ctx, &pin_entry, |pin| {
-      device
-        .factory_reset(&pin)
-        .context("Failed to reset to factory settings")?;
-      // Work around for a timing issue between factory_reset and
-      // build_aes_key, see
-      // https://github.com/Nitrokey/nitrokey-storage-firmware/issues/80
-      thread::sleep(time::Duration::from_secs(3));
-      // Another work around for spurious WrongPassword returns of
-      // build_aes_key after a factory reset on Pro devices.
-      // https://github.com/Nitrokey/nitrokey-pro-firmware/issues/57
-      let _ = device.get_user_retry_count();
-      device
-        .build_aes_key(nitrokey::DEFAULT_ADMIN_PIN)
-        .context("Failed to rebuild AES key")
+      if only_aes_key {
+        // Similar to the else arm, we have to execute this command to avoid WrongPassword errors
+        let _ = device.get_user_retry_count();
+        device
+          .build_aes_key(&pin)
+          .context("Failed to rebuild AES key")
+      } else {
+        device
+          .factory_reset(&pin)
+          .context("Failed to reset to factory settings")?;
+        // Work around for a timing issue between factory_reset and
+        // build_aes_key, see
+        // https://github.com/Nitrokey/nitrokey-storage-firmware/issues/80
+        thread::sleep(time::Duration::from_secs(3));
+        // Another work around for spurious WrongPassword returns of
+        // build_aes_key after a factory reset on Pro devices.
+        // https://github.com/Nitrokey/nitrokey-pro-firmware/issues/57
+        let _ = device.get_user_retry_count();
+        device
+          .build_aes_key(nitrokey::DEFAULT_ADMIN_PIN)
+          .context("Failed to rebuild AES key")
+      }
     })
   })
 }
@@ -795,44 +887,51 @@ fn prepare_ascii_secret(secret: &str) -> anyhow::Result<String> {
 
 /// Prepare a base32 secret string for libnitrokey.
 fn prepare_base32_secret(secret: &str) -> anyhow::Result<String> {
-  base32::decode(base32::Alphabet::RFC4648 { padding: false }, secret)
+  // Some sites display the base32 secret in groups separated by spaces, we want to ignore them.
+  let secret = secret.replace(" ", "");
+  base32::decode(base32::Alphabet::RFC4648 { padding: false }, &secret)
     .map(|vec| format_bytes(&vec))
     .context("Failed to parse base32 secret")
 }
 
-/// Configure a one-time password slot on the Nitrokey device.
-pub fn otp_set(ctx: &mut Context<'_>, mut args: args::OtpSetArgs) -> anyhow::Result<()> {
-  let mut data = nitrokey::OtpSlotData {
-    number: args.slot,
-    name: mem::take(&mut args.name),
-    secret: mem::take(&mut args.secret),
-    mode: args.digits.into(),
-    use_enter: false,
-    token_id: None,
-  };
-
-  with_device(ctx, |ctx, device| {
-    let secret = match args.format {
-      args::OtpSecretFormat::Ascii => prepare_ascii_secret(&data.secret)?,
-      args::OtpSecretFormat::Base32 => prepare_base32_secret(&data.secret)?,
-      args::OtpSecretFormat::Hex => {
-        // We need to ensure to provide a string with an even number of
-        // characters in it, just because that's what libnitrokey
-        // expects. So prepend a '0' if that is not the case.
-        // TODO: This code can be removed once upstream issue #164
-        //       (https://github.com/Nitrokey/libnitrokey/issues/164) is
-        //       addressed.
-        if data.secret.len() % 2 != 0 {
-          data.secret.insert(0, '0')
-        }
-        data.secret
+/// Prepare a secret string in the given format for libnitrokey.
+fn prepare_secret(
+  secret: borrow::Cow<'_, str>,
+  format: args::OtpSecretFormat,
+) -> anyhow::Result<String> {
+  match format {
+    args::OtpSecretFormat::Ascii => prepare_ascii_secret(&secret),
+    args::OtpSecretFormat::Base32 => prepare_base32_secret(&secret),
+    args::OtpSecretFormat::Hex => {
+      // We need to ensure to provide a string with an even number of
+      // characters in it, just because that's what libnitrokey
+      // expects. So prepend a '0' if that is not the case.
+      let mut secret = secret.into_owned();
+      if secret.len() % 2 != 0 {
+        secret.insert(0, '0')
       }
-    };
-    let data = nitrokey::OtpSlotData { secret, ..data };
+      Ok(secret)
+    }
+  }
+}
+
+/// Configure a one-time password slot on the Nitrokey device.
+pub fn otp_set(ctx: &mut Context<'_>, args: args::OtpSetArgs) -> anyhow::Result<()> {
+  // Ideally, we would also like to verify the length of the secret. But the maximum length is
+  // determined by the firmware version of the device and we don't want to run an additional
+  // command just to determine the firmware version.
+  ensure_string_lengths(&[("slot name", &args.name, OTP_NAME_LENGTH)])?;
+
+  let secret = value_or_stdin(ctx, &args.secret)?;
+  let secret = prepare_secret(secret, args.format)?;
+
+  let data = nitrokey::OtpSlotData::new(args.slot, args.name, secret, args.digits.into());
+  let (algorithm, counter, time_window) = (args.algorithm, args.counter, args.time_window);
+  with_device(ctx, |ctx, device| {
     let mut device = authenticate_admin(ctx, device)?;
-    match args.algorithm {
-      args::OtpAlgorithm::Hotp => device.write_hotp_slot(data, args.counter),
-      args::OtpAlgorithm::Totp => device.write_totp_slot(data, args.time_window),
+    match algorithm {
+      args::OtpAlgorithm::Hotp => device.write_hotp_slot(data, counter),
+      args::OtpAlgorithm::Totp => device.write_totp_slot(data, time_window),
     }
     .context("Failed to write OTP slot")?;
     Ok(())
@@ -996,20 +1095,6 @@ fn print_pws_data(
   Ok(())
 }
 
-fn check_slot(pws: &nitrokey::PasswordSafe<'_, '_>, slot: u8) -> anyhow::Result<()> {
-  if slot >= nitrokey::SLOT_COUNT {
-    anyhow::bail!("Slot {} is not valid", slot);
-  }
-  let status = pws
-    .get_slot_status()
-    .context("Failed to read PWS slot status")?;
-  if status[slot as usize] {
-    Ok(())
-  } else {
-    anyhow::bail!("Slot {} is not programmed", slot)
-  }
-}
-
 /// Read a PWS slot.
 pub fn pws_get(
   ctx: &mut Context<'_>,
@@ -1020,33 +1105,119 @@ pub fn pws_get(
   quiet: bool,
 ) -> anyhow::Result<()> {
   with_password_safe(ctx, |ctx, pws| {
-    check_slot(&pws, slot).context("Failed to access PWS slot")?;
+    let slot = pws.get_slot(slot).context("Failed to access PWS slot")?;
 
     let show_all = !show_name && !show_login && !show_password;
     if show_all || show_name {
-      print_pws_data(ctx, "name:    ", pws.get_slot_name(slot), quiet)?;
+      print_pws_data(ctx, "name:    ", slot.get_name(), quiet)?;
     }
     if show_all || show_login {
-      print_pws_data(ctx, "login:   ", pws.get_slot_login(slot), quiet)?;
+      print_pws_data(ctx, "login:   ", slot.get_login(), quiet)?;
     }
     if show_all || show_password {
-      print_pws_data(ctx, "password:", pws.get_slot_password(slot), quiet)?;
+      print_pws_data(ctx, "password:", slot.get_password(), quiet)?;
     }
     Ok(())
   })
 }
 
-/// Write a PWS slot.
-pub fn pws_set(
+fn ensure_pws_string_lengths(
+  name: Option<&str>,
+  login: Option<&str>,
+  password: Option<&str>,
+) -> anyhow::Result<()> {
+  let mut data = Vec::new();
+  if let Some(name) = name {
+    data.push(("slot name", name, PWS_NAME_LENGTH));
+  }
+  if let Some(login) = login {
+    data.push(("login", login, PWS_LOGIN_LENGTH));
+  }
+  if let Some(password) = password {
+    data.push(("password", password, PWS_PASSWORD_LENGTH));
+  }
+  ensure_string_lengths(&data)
+}
+
+/// Add a new PWS slot.
+pub fn pws_add(
   ctx: &mut Context<'_>,
-  slot: u8,
   name: &str,
   login: &str,
   password: &str,
+  slot_idx: Option<u8>,
 ) -> anyhow::Result<()> {
-  with_password_safe(ctx, |_ctx, mut pws| {
+  let password = value_or_stdin(ctx, password)?;
+  ensure_pws_string_lengths(Some(name), Some(login), Some(&password))?;
+  with_password_safe(ctx, |ctx, mut pws| {
+    let slots = pws.get_slots()?;
+
+    let slot_idx = if let Some(slot_idx) = slot_idx {
+      // If the user specified a slot, make sure that it is not programmed
+      if let Some(slot) = slots.get(usize::from(slot_idx)) {
+        if slot.is_some() {
+          Err(anyhow::anyhow!(
+            "The PWS slot {} is already programmed",
+            slot_idx
+          ))
+        } else {
+          Ok(slot_idx)
+        }
+      } else {
+        Err(anyhow::anyhow!(
+          "Encountered invalid slot index: {}",
+          slot_idx
+        ))
+      }
+    } else {
+      // If the user did not specify a slot, we try to find the first unprogrammed slot
+      if let Some(slot_idx) = slots.iter().position(Option::is_none) {
+        u8::try_from(slot_idx).context("Unexpected number of PWS slots")
+      } else {
+        Err(anyhow::anyhow!("All PWS slots are already programmed"))
+      }
+    }?;
+
     pws
-      .write_slot(slot, name, login, password)
+      .write_slot(slot_idx, name, login, password.as_ref())
+      .context("Failed to write PWS slot")?;
+    println!(ctx, "Added PWS slot {}", slot_idx)?;
+    Ok(())
+  })
+}
+
+/// Update a PWS slot.
+pub fn pws_update(
+  ctx: &mut Context<'_>,
+  slot_idx: u8,
+  name: Option<&str>,
+  login: Option<&str>,
+  password: Option<&str>,
+) -> anyhow::Result<()> {
+  if name.is_none() && login.is_none() && password.is_none() {
+    anyhow::bail!("You have to set at least one of --name, --login, or --password");
+  }
+
+  let password = password.map(|s| value_or_stdin(ctx, s)).transpose()?;
+  ensure_pws_string_lengths(name, login, password.as_deref())?;
+
+  with_password_safe(ctx, |_ctx, mut pws| {
+    let slot = pws.get_slot(slot_idx).context("Failed to query PWS slot")?;
+    let name = name
+      .map(|s| Ok(borrow::Cow::from(s)))
+      .unwrap_or_else(|| slot.get_name().map(borrow::Cow::from))
+      .context("Failed to query current slot name")?;
+    let login = login
+      .map(|s| Ok(borrow::Cow::from(s)))
+      .unwrap_or_else(|| slot.get_login().map(borrow::Cow::from))
+      .context("Failed to query current slot login")?;
+    let password = password
+      .as_ref()
+      .map(|s| Ok(borrow::Cow::from(s.as_ref())))
+      .unwrap_or_else(|| slot.get_password().map(borrow::Cow::from))
+      .context("Failed to query current slot password")?;
+    pws
+      .write_slot(slot_idx, name.as_ref(), login.as_ref(), password.as_ref())
       .context("Failed to write PWS slot")
   })
 }
@@ -1060,31 +1231,29 @@ pub fn pws_clear(ctx: &mut Context<'_>, slot: u8) -> anyhow::Result<()> {
 
 fn print_pws_slot(
   ctx: &mut Context<'_>,
-  pws: &nitrokey::PasswordSafe<'_, '_>,
-  slot: usize,
-  programmed: bool,
+  index: usize,
+  slot: Option<nitrokey::PasswordSlot<'_, '_, '_>>,
 ) -> anyhow::Result<()> {
-  let slot = u8::try_from(slot).map_err(|_| anyhow::anyhow!("Invalid PWS slot number"))?;
-  let name = if programmed {
-    pws
-      .get_slot_name(slot)
-      .context("Failed to read PWS slot name")?
+  let name = if let Some(slot) = slot {
+    slot.get_name().context("Failed to read PWS slot name")?
   } else {
     "[not programmed]".to_string()
   };
-  println!(ctx, "{}\t{}", slot, name)?;
+  println!(ctx, "{}\t{}", index, name)?;
   Ok(())
 }
 
 /// Print the status of all PWS slots.
 pub fn pws_status(ctx: &mut Context<'_>, all: bool) -> anyhow::Result<()> {
   with_password_safe(ctx, |ctx, pws| {
-    let slots = pws
-      .get_slot_status()
-      .context("Failed to read PWS slot status")?;
+    let slots = pws.get_slots().context("Failed to read PWS slot status")?;
     println!(ctx, "slot\tname")?;
-    for (i, &value) in slots.iter().enumerate().filter(|(_, &value)| all || value) {
-      print_pws_slot(ctx, &pws, i, value)?;
+    for (i, &slot) in slots
+      .iter()
+      .enumerate()
+      .filter(|(_, &slot)| all || slot.is_some())
+    {
+      print_pws_slot(ctx, i, slot)?;
     }
     Ok(())
   })
@@ -1185,6 +1354,10 @@ pub fn extension(ctx: &mut Context<'_>, args: Vec<ffi::OsString>) -> anyhow::Res
   // a cargo test context.
   let mut cmd = process::Command::new(&ext_path);
 
+  if let Ok(device_info) = find_device(&ctx.config) {
+    let _ = cmd.env(crate::NITROCLI_RESOLVED_USB_PATH, device_info.path);
+  }
+
   if let Some(model) = ctx.config.model {
     let _ = cmd.env(crate::NITROCLI_MODEL, model.to_string());
   }
@@ -1243,6 +1416,17 @@ mod tests {
     assert!(result.is_err());
   }
 
+  #[test]
+  fn prepare_secret_base32() {
+    let result = prepare_base32_secret("gezdgnbvgy3tqojqgezdgnbvgy3tqojq").unwrap();
+    assert_eq!(
+      "3132333435363738393031323334353637383930".to_string(),
+      result
+    );
+    let result2 = prepare_base32_secret("gezd gnbv     gy3t qojq gezd gnbv gy3t qojq").unwrap();
+    assert_eq!(result, result2);
+  }
+
   #[test]
   fn hex_string() {
     assert_eq!(format_bytes(&[b' ']), "20");
