@@ -74,6 +74,7 @@ use structopt::clap::SubCommand;
 use structopt::StructOpt;
 
 const NITROCLI_BINARY: &str = "NITROCLI_BINARY";
+const NITROCLI_RESOLVED_USB_PATH: &str = "NITROCLI_RESOLVED_USB_PATH";
 const NITROCLI_MODEL: &str = "NITROCLI_MODEL";
 const NITROCLI_USB_PATH: &str = "NITROCLI_USB_PATH";
 const NITROCLI_VERBOSITY: &str = "NITROCLI_VERBOSITY";
@@ -110,8 +111,11 @@ impl error::Error for DirectExitError {}
 
 /// Parse the command-line arguments and execute the selected command.
 fn handle_arguments(ctx: &mut Context<'_>, argv: Vec<String>) -> anyhow::Result<()> {
-  match args::Args::from_iter_safe(argv.iter()) {
-    Ok(args) => {
+  let version = get_version_string();
+  let clap = args::Args::clap().version(version.as_str());
+  match clap.get_matches_from_safe(argv.iter()) {
+    Ok(matches) => {
+      let args = args::Args::from_clap(&matches);
       ctx.config.update(&args);
       args.cmd.execute(ctx)
     }
@@ -157,9 +161,20 @@ fn handle_arguments(ctx: &mut Context<'_>, argv: Vec<String>) -> anyhow::Result<
   }
 }
 
+fn get_version_string() -> String {
+  let version = env!("CARGO_PKG_VERSION");
+  if let Ok(library_version) = nitrokey::get_library_version() {
+    format!("{} using libnitrokey {}", version, library_version)
+  } else {
+    format!("{} using an undetectable libnitrokey version", version)
+  }
+}
+
 /// The context used when running the program.
 #[allow(missing_debug_implementations)]
 pub struct Context<'io> {
+  /// The `Read` object used as standard input throughout the program.
+  pub stdin: &'io mut dyn io::Read,
   /// The `Write` object used as standard output throughout the program.
   pub stdout: &'io mut dyn io::Write,
   /// The `Write` object used as standard error throughout the program.
@@ -188,17 +203,20 @@ pub struct Context<'io> {
 }
 
 impl<'io> Context<'io> {
-  fn from_env<O, E>(
+  fn from_env<I, O, E>(
+    stdin: &'io mut I,
     stdout: &'io mut O,
     stderr: &'io mut E,
     is_tty: bool,
     config: config::Config,
   ) -> Context<'io>
   where
+    I: io::Read,
     O: io::Write,
     E: io::Write,
   {
     Context {
+      stdin,
       stdout,
       stderr,
       is_tty,
@@ -220,7 +238,7 @@ fn evaluate_err(err: anyhow::Error, stderr: &mut dyn io::Write) -> i32 {
   if let Some(err) = err.root_cause().downcast_ref::<DirectExitError>() {
     err.0
   } else {
-    let _ = writeln!(stderr, "{:?}", err);
+    let _ = writeln!(stderr, "{:#}", err);
     1
   }
 }
@@ -234,6 +252,7 @@ fn run<'ctx, 'io: 'ctx>(ctx: &'ctx mut Context<'io>, args: Vec<String>) -> i32 {
 fn main() {
   use std::io::Write;
 
+  let mut stdin = io::stdin();
   let mut stdout = io::stdout();
   let mut stderr = io::stderr();
 
@@ -241,7 +260,7 @@ fn main() {
     Ok(config) => {
       let is_tty = termion::is_tty(&stdout);
       let args = env::args().collect::<Vec<_>>();
-      let ctx = &mut Context::from_env(&mut stdout, &mut stderr, is_tty, config);
+      let ctx = &mut Context::from_env(&mut stdin, &mut stdout, &mut stderr, is_tty, config);
 
       run(ctx, args)
     }
