@@ -23,6 +23,6 @@ fn connected(model: nitrokey::Model) -> anyhow::Result<()> {
   .unwrap();
 
   let out = Nitrocli::new().model(model).handle(&["list"])?;
-  assert!(re.is_match(&out), out);
+  assert!(re.is_match(&out), "{}", out);
   Ok(())
 }
