@@ -1,6 +1,6 @@
 // otp.rs
 
-// Copyright (C) 2019-2020 The Nitrocli Developers
+// Copyright (C) 2019-2021 The Nitrocli Developers
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 use super::*;
@@ -14,8 +14,13 @@ fn set_invalid_slot_raw(model: nitrokey::Model) {
     .run(&["otp", "set", "100", "name", "1234", "-f", "hex"]);
 
   assert_ne!(rc, 0);
-  assert_eq!(out, b"");
-  assert_eq!(&err[..24], b"Failed to write OTP slot");
+  assert_eq!(out, b"", "{}", String::from_utf8_lossy(&out));
+  assert_eq!(
+    &err[..24],
+    b"Failed to write OTP slot",
+    "{}",
+    String::from_utf8_lossy(&err)
+  );
 }
 
 #[test_device]
@@ -29,6 +34,39 @@ fn set_invalid_slot(model: nitrokey::Model) {
   assert_eq!(err, "Failed to write OTP slot");
 }
 
+#[test_device]
+fn set_overlong_name(model: nitrokey::Model) {
+  let err = Nitrocli::new()
+    .model(model)
+    .handle(&["otp", "set", "0", "1234567890123456", "1234"])
+    .unwrap_err()
+    .to_string();
+  assert_eq!(
+    err,
+    "The provided slot name is too long (actual length: 16 bytes, maximum length: 15 bytes)"
+  );
+
+  let err = Nitrocli::new()
+    .model(model)
+    .handle(&["otp", "set", "0", "ö23456789012345", "1234"])
+    .unwrap_err()
+    .to_string();
+  assert_eq!(
+    err,
+    "The provided slot name is too long (actual length: 16 bytes, maximum length: 15 bytes)"
+  );
+
+  let err = Nitrocli::new()
+    .model(model)
+    .handle(&["otp", "set", "0", "1234567890123456789012345", "1234"])
+    .unwrap_err()
+    .to_string();
+  assert_eq!(
+    err,
+    "The provided slot name is too long (actual length: 25 bytes, maximum length: 15 bytes)"
+  );
+}
+
 #[test_device]
 fn status(model: nitrokey::Model) -> anyhow::Result<()> {
   let re = regex::Regex::new(
@@ -43,7 +81,7 @@ fn status(model: nitrokey::Model) -> anyhow::Result<()> {
   let _ = ncli.handle(&["otp", "set", "0", "the-name", "123456", "-f", "hex"])?;
 
   let out = ncli.handle(&["otp", "status"])?;
-  assert!(re.is_match(&out), out);
+  assert!(re.is_match(&out), "{}", out);
   Ok(())
 }
 
@@ -98,6 +136,24 @@ fn set_totp_uneven_chars(model: nitrokey::Model) -> anyhow::Result<()> {
   Ok(())
 }
 
+#[test_device]
+fn set_stdin(model: nitrokey::Model) -> anyhow::Result<()> {
+  const SECRET: &str = "12345678901234567890";
+  const TIME: &str = stringify!(1111111111);
+  const OTP: &str = concat!(14050471, "\n");
+
+  let _ = Nitrocli::new()
+    .model(model)
+    .stdin(SECRET)
+    .handle(&["otp", "set", "-d", "8", "-f", "ascii", "2", "name", "-"])?;
+
+  let out = Nitrocli::new()
+    .model(model)
+    .handle(&["otp", "get", "-t", TIME, "2"])?;
+  assert_eq!(out, OTP);
+  Ok(())
+}
+
 #[test_device]
 fn clear(model: nitrokey::Model) -> anyhow::Result<()> {
   let mut ncli = Nitrocli::new().model(model);
