@@ -1,21 +1,7 @@
 // pin.rs
 
-// *************************************************************************
-// * Copyright (C) 2019-2020 Daniel Mueller (deso@posteo.net)              *
-// *                                                                       *
-// * This program is free software: you can redistribute it and/or modify  *
-// * it under the terms of the GNU General Public License as published by  *
-// * the Free Software Foundation, either version 3 of the License, or     *
-// * (at your option) any later version.                                   *
-// *                                                                       *
-// * This program is distributed in the hope that it will be useful,       *
-// * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-// * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-// * GNU General Public License for more details.                          *
-// *                                                                       *
-// * You should have received a copy of the GNU General Public License     *
-// * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
-// *************************************************************************
+// Copyright (C) 2019-2020 The Nitrocli Developers
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 use nitrokey::Authenticate;
 use nitrokey::Device;
@@ -35,7 +21,7 @@ fn unblock(model: nitrokey::Model) -> anyhow::Result<()> {
     assert!(device.get_user_retry_count()? < 3);
   }
 
-  let _ = Nitrocli::with_model(model).handle(&["pin", "unblock"])?;
+  let _ = Nitrocli::new().model(model).handle(&["pin", "unblock"])?;
 
   {
     let mut manager = nitrokey::force_take()?;
@@ -47,9 +33,9 @@ fn unblock(model: nitrokey::Model) -> anyhow::Result<()> {
 
 #[test_device]
 fn set_user(model: nitrokey::Model) -> anyhow::Result<()> {
-  let mut ncli = Nitrocli::with_model(model);
+  let ncli = Nitrocli::new().model(model);
   // Set a new user PIN.
-  ncli.new_user_pin("new-pin");
+  let mut ncli = ncli.new_user_pin("new-pin");
   let out = ncli.handle(&["pin", "set", "user"])?;
   assert!(out.is_empty());
 
@@ -67,15 +53,16 @@ fn set_user(model: nitrokey::Model) -> anyhow::Result<()> {
   }
 
   // Revert to the default user PIN.
-  ncli.user_pin("new-pin");
-  ncli.new_user_pin(nitrokey::DEFAULT_USER_PIN);
+  let mut ncli = ncli
+    .user_pin("new-pin")
+    .new_user_pin(nitrokey::DEFAULT_USER_PIN);
 
   let out = ncli.handle(&["pin", "set", "user"])?;
   assert!(out.is_empty());
 
   {
     let mut manager = nitrokey::force_take()?;
-    let device = manager.connect_model(ncli.model().unwrap())?;
+    let device = manager.connect_model(model)?;
     let _ = device
       .authenticate_user(nitrokey::DEFAULT_USER_PIN)
       .unwrap();
