@@ -1,6 +1,6 @@
 // run.rs
 
-// Copyright (C) 2019-2020 The Nitrocli Developers
+// Copyright (C) 2019-2021 The Nitrocli Developers
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 use std::collections;
@@ -21,11 +21,11 @@ fn no_command_or_option() {
   let (rc, out, err) = Nitrocli::new().run(&[]);
 
   assert_ne!(rc, 0);
-  assert_eq!(out, b"");
+  assert_eq!(out, b"", "{}", String::from_utf8_lossy(&out));
 
   let s = String::from_utf8_lossy(&err).into_owned();
-  assert!(s.starts_with("nitrocli"), s);
-  assert!(s.contains("USAGE:\n"), s);
+  assert!(s.starts_with("nitrocli"), "{}", s);
+  assert!(s.contains("USAGE:\n"), "{}", s);
 }
 
 #[test]
@@ -37,13 +37,13 @@ fn help_options() {
     let (rc, out, err) = Nitrocli::new().run(&all);
 
     assert_eq!(rc, 0);
-    assert_eq!(err, b"");
+    assert_eq!(err, b"", "{}", String::from_utf8_lossy(&err));
 
     let s = String::from_utf8_lossy(&out).into_owned();
     let mut args = args.to_vec();
     args.insert(0, "nitrocli");
-    assert!(s.starts_with(&args.join("-")), s);
-    assert!(s.contains("USAGE:\n"), s);
+    assert!(s.starts_with(&args.join("-")), "{}", s);
+    assert!(s.contains("USAGE:\n"), "{}", s);
   }
 
   fn test(args: &[&str]) {
@@ -75,7 +75,8 @@ fn help_options() {
   test(&["pws"]);
   test(&["pws", "clear"]);
   test(&["pws", "get"]);
-  test(&["pws", "set"]);
+  test(&["pws", "add"]);
+  test(&["pws", "update"]);
   test(&["pws", "status"]);
   test(&["reset"]);
   test(&["status"]);
@@ -93,14 +94,14 @@ fn version_option() {
     let (rc, out, err) = Nitrocli::new().run(&[opt]);
 
     assert_eq!(rc, 0);
-    assert_eq!(err, b"");
+    assert_eq!(err, b"", "{}", String::from_utf8_lossy(&err));
 
     let s = String::from_utf8_lossy(&out).into_owned();
     let _ = re;
-    assert!(re.is_match(&s), out);
+    assert!(re.is_match(&s), "{}", s);
   }
 
-  let re = regex::Regex::new(r"^nitrocli \d+.\d+.\d+(-[^-]+)*\n$").unwrap();
+  let re = regex::Regex::new(r"^nitrocli \d+.\d+.\d+(-[^-]+)* using libnitrokey .*\n$").unwrap();
 
   test(&re, "--version");
   test(&re, "-V");
@@ -302,7 +303,11 @@ print("success")
   let path = ext_dir.path().as_os_str().to_os_string();
   // Make sure that the extension appears in the help text.
   let out = Nitrocli::new().path(&path).handle(&["--help"])?;
-  assert!(out.contains("ext            Run the ext extension\n"), out);
+  assert!(
+    out.contains("ext            Run the ext extension\n"),
+    "{}",
+    out
+  );
   // And, of course, that we can invoke it.
   let out = Nitrocli::new().path(&path).handle(&["ext"])?;
   assert_eq!(out, "success\n");
@@ -342,8 +347,8 @@ sys.exit(42);
 
   let (rc, out, err) = ncli.run(&["ext"]);
   assert_eq!(rc, 42);
-  assert_eq!(out, b"");
-  assert_eq!(err, b"");
+  assert_eq!(out, b"", "{}", String::from_utf8_lossy(&out));
+  assert_eq!(err, b"", "{}", String::from_utf8_lossy(&err));
   Ok(())
 }
 
@@ -370,7 +375,7 @@ fn extension_arguments(model: nitrokey::Model) -> anyhow::Result<()> {
     let path = ext_dir.path().as_os_str().to_os_string();
     let out = Nitrocli::new().model(model).path(path).handle(&args)?;
 
-    assert!(check(&out), out);
+    assert!(check(&out), "{}", out);
     Ok(())
   }
 
